@@ -4,15 +4,16 @@
 
 """ Public XML parser module """
 
-import lxml.etree
 import re
 import os
+import codecs
 import shutil
+import six
+import time
+import requests
+import lxml.etree
 import xml2rfc.log
 import xml2rfc.utils
-import requests
-import codecs
-import time
 
 try:
     from urllib.parse import urlparse, urljoin
@@ -45,9 +46,10 @@ class CachingResolver(lxml.etree.Resolver):
     def __init__(self, cache_path=None, library_dirs=None, source=None,
                  templates_path='templates', verbose=False, quiet=False,
                  no_network=False, network_locs= [
+                     'https://xml2rfc.ietf.org/public/rfc/',
+                     'https://xml2rfc.tools.ietf.org/public/rfc/',
                      'http://xml2rfc.ietf.org/public/rfc/',
                      'http://xml2rfc.tools.ietf.org/public/rfc/',
-                     'http://xml.resource.org/public/rfc/',
                  ],
                  rfc_number=None):
         self.verbose = verbose
@@ -63,7 +65,10 @@ class CachingResolver(lxml.etree.Resolver):
 
         # Get directory of source
         if self.source:
-            self.source_dir = os.path.abspath(os.path.dirname(self.source))
+            if isinstance(self.source, six.string_types):
+                self.source_dir = os.path.abspath(os.path.dirname(self.source))
+            else:
+                self.source_dir = os.path.abspath(os.path.dirname(self.source.name))                
         else:
             self.source_dir = None
 
@@ -142,11 +147,12 @@ class CachingResolver(lxml.etree.Resolver):
             If REQUEST ends with '.dtd' or '.ent' then
               If REQUEST is an absolute path (local or network) then
                 Return REQUEST
+              Else
+                Try TEMPLATE_DIR + REQUEST, otherwise
+                Return SOURCE_DIR + REQUEST
             Else
-              Try TEMPLATE_DIR + REQUEST, otherwise
-              Return SOURCE_DIR + REQUEST
-            Else
-              If REQUEST doesn't end with '.xml' then append '.xml'
+              If REQUEST doesn't end with '.xml' then
+                append '.xml'
               If REQUEST is an absolute path (local or network) then
                 Return REQUEST
               Else
@@ -168,7 +174,7 @@ class CachingResolver(lxml.etree.Resolver):
               ['/usr/share/xml2rfc'].  On the GUI this can be configured
               manually but has the same initial defaults.
             - NETWORK refers to the online citation library.  On the CLI this
-              is http://xml.resource.org/public/rfc/.  On the GUI this 
+              is http://xml2rfc.ietf.org/public/rfc/.  On the GUI this 
               can be configured manually but has the same initial default.
 
             The caches in read_dirs are consulted in sequence order to find the
@@ -358,15 +364,30 @@ class CachingResolver(lxml.etree.Resolver):
             return ''
 
 
+class AnnotatedElement(lxml.etree.ElementBase):
+    pis = None
+    def get(self, key, default=None):
+        value = super(AnnotatedElement, self).get(key, default)
+        if value == default:
+            return value
+        else:
+            return six.text_type(value)
+
 class XmlRfcParser:
 
+    nsmap = {
+        'xi':   'http://www.w3.org/2001/XInclude',
+    }
+
+
     """ XML parser container with callbacks to construct an RFC tree """
-    def __init__(self, source, verbose=False, quiet=False,
+    def __init__(self, source, verbose=False, quiet=False, options=None,
                  cache_path=None, templates_path=None, library_dirs=None,
                  no_network=False, network_locs=[
+                     'https://xml2rfc.ietf.org/public/rfc/',
+                     'https://xml2rfc.tools.ietf.org/public/rfc/',
                      'http://xml2rfc.ietf.org/public/rfc/',
                      'http://xml2rfc.tools.ietf.org/public/rfc/',
-                     'http://xml.resource.org/public/rfc/',
                  ]
                  ):
         self.verbose = verbose
@@ -376,11 +397,25 @@ class XmlRfcParser:
         self.no_network = no_network
         self.network_locs = network_locs
 
+        if self.source:
+            if six.PY2:
+                with open(self.source, "rU") as f:
+                    self.text = f.read()
+            else:
+                with open(self.source, "rb", newline=None) as f:
+                    self.text = f.read()
+
         # Initialize templates directory
         self.templates_path = templates_path or \
                               os.path.join(os.path.dirname(xml2rfc.__file__),
                                            'templates')
-        self.default_dtd_path = os.path.join(self.templates_path, 'rfc2629.dtd')
+        if options and options.vocabulary == 'v2':
+            self.default_dtd_path = os.path.join(self.templates_path, 'rfc2629.dtd')
+        else:
+            self.default_dtd_path = None
+
+        for prefix, value in self.nsmap.items():
+            lxml.etree.register_namespace(prefix, value)
 
         # If library dirs werent explicitly set, like from the gui, then try:
         #   1. $XML_LIBRARY environment variable as a delimited list
@@ -412,13 +447,14 @@ class XmlRfcParser:
     def delete_cache(self, path=None):
         self.cachingResolver.delete_cache(path=path)
 
-    def parse(self, remove_comments=True, remove_pis=False, quiet=False, strip_cdata=True):
+    def parse(self, remove_comments=True, remove_pis=False, quiet=False, strip_cdata=True, normalize=False):
         """ Parses the source XML file and returns an XmlRfc instance """
         if not (self.quiet or quiet):
             xml2rfc.log.write('Parsing file', self.source)
 
         # Get an iterating parser object
-        context = lxml.etree.iterparse(self.source,
+        file = six.BytesIO(self.text)
+        context = lxml.etree.iterparse(file,
                                       dtd_validation=False,
                                       load_dtd=True,
                                       attribute_defaults=True,
@@ -440,7 +476,7 @@ class XmlRfcParser:
                                         network_locs=self.network_locs,
                                         verbose=self.verbose,
                                         quiet=self.quiet,
-                                    )
+                                     )
         context.resolvers.add(caching_resolver)
 
         # Get hold of the rfc number (if any) in the rfc element, so we can
@@ -450,8 +486,12 @@ class XmlRfcParser:
             for action, element in context:
                 if element.tag == "rfc":
                     self.rfc_number = element.attrib.get("number", None)
+                    break
         except lxml.etree.XMLSyntaxError as e:
             xml2rfc.log.warn("Parsing Error: %s" % e)
+        except ValueError as e:
+            if e.message=="I/O operation on closed file":
+                pass
 
         # now get a regular parser, and parse again, this time resolving entities
         parser = lxml.etree.XMLParser(dtd_validation=False,
@@ -479,17 +519,31 @@ class XmlRfcParser:
         # Add our custom resolver
         parser.resolvers.add(self.cachingResolver)
 
+        # Use our custom element class, which holds the state of PI settings
+        # at this point in the xml tree
+        element_lookup = lxml.etree.ElementDefaultClassLookup(element=AnnotatedElement)
+        parser.set_element_class_lookup(element_lookup)
+
         # Parse the XML file into a tree and create an rfc instance
-        tree = lxml.etree.parse(self.source, parser)
-        xmlrfc = XmlRfc(tree, self.default_dtd_path)
-        
-        # Evaluate processing instructions behind root element
+        file = six.BytesIO(self.text)
+        tree = lxml.etree.parse(file, parser)
+        xmlrfc = XmlRfc(tree, self.default_dtd_path, nsmap=self.nsmap)
+        xmlrfc.source = self.source
+
+        # Evaluate processing instructions before root element
         xmlrfc._eval_pre_pi()
         
-        # Expand 'include' instructions
-        for element in xmlrfc.getroot().iter():
+        # Keep seen elements in a list, to force lxml to not discard (and
+        # recreate) the elements, as this would cause loss of our custom
+        # state, the PI settings at the time the element was parsed
+        # (in element.pis)
+        xmlrfc._elements_cache = []
+        # Process PIs and expand 'include' instructions
+        pis = xmlrfc.pis.copy()
+        for element in xmlrfc.getroot().iterdescendants():
             if element.tag is lxml.etree.PI:
                 pidict = xmlrfc.parse_pi(element)
+                pis = xmlrfc.pis.copy() 
                 if 'include' in pidict and pidict['include']:
                     request = pidict['include']
                     path = self.cachingResolver.getReferenceRequest(request,
@@ -504,8 +558,14 @@ class XmlRfcParser:
                                                       remove_blank_text=True,
                                                       resolve_entities=True,
                                                       strip_cdata=strip_cdata)
+                        parser.set_element_class_lookup(element_lookup)
                         # parser.resolvers.add(self.cachingResolver) --- should this be done?
                         ref_root = lxml.etree.parse(path, parser).getroot()
+                        ref_root.pis = pis
+                        xmlrfc._elements_cache.append(ref_root)
+                        for e in ref_root.iterdescendants():
+                            e.pis = pis
+                            xmlrfc._elements_cache.append(e)
                         parent = element.getparent()
                         parent.replace(element, ref_root)
                     except (lxml.etree.XMLSyntaxError, IOError) as e:
@@ -516,9 +576,14 @@ class XmlRfcParser:
                         else:
                             xml2rfc.log.warn('Unable to load the include file at',
                                               path)
+            else:
+                if isinstance(element, AnnotatedElement):
+                    element.pis = pis
+                    xmlrfc._elements_cache.append(element)                    
 
         # Finally, do any extra formatting on the RFC before returning
-        xmlrfc._format_whitespace()
+        if normalize:
+            xmlrfc._format_whitespace()
 
         return xmlrfc
 
@@ -531,13 +596,16 @@ class XmlRfc:
         Accessing the rfc tree is done by getting the root node from getroot()
     """
 
-    def __init__(self, tree, default_dtd_path):
+    pis = {}
+
+    def __init__(self, tree, default_dtd_path, nsmap={}):
         self.default_dtd_path = default_dtd_path
         self.tree = tree
+        self.nsmap = nsmap
         # Pi default values
         self.pis = {
             "artworkdelimiter":	None,
-            "artworklines":	0 ,
+            "artworklines":	"0",
             "authorship":	"yes",
             "autobreaks":	"yes",
             "background":	"" ,
@@ -545,37 +613,40 @@ class XmlRfc:
             "comments":		"no" ,
             "docmapping":	"no",
             "editing":		"no",
-            "emoticonic":	"no",
+            #"emoticonic":	"no",
             #"footer":		Unset
             "figurecount":      "no",
             #"header":		Unset
             "inline":		"no",
-            "iprnotified":	"no",
+            #"iprnotified":	"no",
             "linkmailto":	"yes",
             #"linefile":	Unset
-            #"needLines":       Unset
+            "needLines":        None,
             "multiple-initials":"no",
-            "notedraftinprogress": "yes",
+            #"notedraftinprogress": "yes",
+            "orphanlimit":      "2",
             "private":		"",
             "refparent":	"References",
             "rfcedstyle":	"no",
-            "rfcprocack":	"no",
-            "sectionorphan":    4,
-            "slides":		"no",
-            "sortrefs":		"no",
-            "strict":		"no",
+            #"rfcprocack":	"no",
+            "sectionorphan":    "4",
+            #"slides":		"no",
+            "sortrefs":		"yes",
+            #"strict":		"no",
             "symrefs":		"yes",
             "tablecount":       "no",
             "text-list-symbols": "o*+-",
             "toc":		"no",
             "tocappendix":	"yes",
-            "tocdepth":		3,
+            "tocdepth":		"3",
             "tocindent":	"yes",
             "tocnarrow":	"yes",
-            "tocompact":	"yes",
+            #"tocompact":	"yes",
+            "tocpagebreak":     "no",
             "topblock":		"yes",
             #"typeout":		Unset
-            "useobject":	"no" ,
+            #"useobject":	"no" ,
+            "widowlimit":       "2",
         }
         # Special cases:
         self.pis["compact"] = self.pis["rfcedstyle"]
@@ -613,12 +684,12 @@ class XmlRfc:
         else:
             dtd = self.tree.docinfo.externalDTD
             
-        if not dtd:
+        if not dtd and self.default_dtd_path:
             # No explicit DTD filename OR declaration in document!
             xml2rfc.log.warn('No DTD given, defaulting to', self.default_dtd_path)
             return self.validate(dtd_path=self.default_dtd_path)
 
-        if dtd.validate(self.getroot()):
+        if not dtd or dtd.validate(self.getroot()):
             # The document was valid
             return True, []
         else:
