@@ -4,6 +4,8 @@
 
 # Python libs
 import datetime
+import calendar
+
 try:
     import debug
     assert debug
@@ -11,7 +13,7 @@ except ImportError:
     pass
 
 # Local libs
-from xml2rfc.writers.base import BaseRfcWriter
+from xml2rfc.writers.base import BaseRfcWriter, default_options
 from xml2rfc.writers.raw_txt import RawTextRfcWriter
 import xml2rfc.utils
 import xml2rfc.log
@@ -22,10 +24,11 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
         The page width is controlled by the *width* parameter.
     """
 
-    def __init__(self, xmlrfc, width=72, quiet=False, verbose=False, date=datetime.date.today(),
-                    omit_headers=False):
-        RawTextRfcWriter.__init__(self, xmlrfc, width=width, quiet=quiet, \
-                                  verbose=verbose, date=date)
+    def __init__(self, xmlrfc, width=72, quiet=None, options=default_options,
+                               date=datetime.date.today(), omit_headers=False):
+        if not quiet is None:
+            options.quiet = quiet
+        RawTextRfcWriter.__init__(self, xmlrfc, options=options, date=date)
         self.left_header = ''
         self.center_header = ''
         self.right_header = ''
@@ -36,6 +39,13 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
         self.paged_toc_marker = 0
         self.page_line = 0
         self.omit_headers = omit_headers
+        self.page_end_blank_lines = 2
+        # Don't permit less than this many lines of a broken paragrah at the
+        # top of a page:
+        self.widow_limit = self.get_numeric_pi('widowlimit', default=2)
+        # Don't permit less than this many lines of a broken paragrah at the
+        # bottom of a page:
+        self.orphan_limit = self.get_numeric_pi('orphanlimit', default=2)
 
     def _make_footer_and_header(self, page, final=False):
         tmp = []
@@ -149,9 +159,18 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
                                        level=level)
         # Reserve room for a blankline and some lines of section content
         # text, in order to prevent orphan headings
-        end = len(self.buf) + self.pis["sectionorphan"]
-        self._set_break_hint(end-begin, 'raw', begin)
-                                        
+        #
+        # We've now written a blank line before the heading, followed by
+        # the heading.  Next will be a blank line, followed by the section
+        # content.  We thus need (begin - end) + 1 + orphan_limit lines, but
+        # if we ask for only that, we'll end up moving the text, to the next
+        # page, without moving the section heading.  So we ask for one more
+        self.orphan_limit = self.get_numeric_pi('orphanlimit', default=self.orphan_limit)
+        orphanlines = self.get_numeric_pi('sectionorphan', default=self.orphan_limit+2)
+        end = len(self.buf)
+        needed = end - begin + orphanlines
+        self._set_break_hint(needed, 'raw', begin)
+
 
     def pre_rendering(self):
         """ Prepares the header and footer information """
@@ -174,6 +193,8 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
         date = self.r.find('front/date')
         if date is not None:
             month = date.attrib.get('month', '')
+            if month.isdigit():
+                month = calendar.month_name[int(month)]
             year = date.attrib.get('year', '')
             self.right_header = month + ' ' + year
         authors = self.r.findall('front/author')
@@ -255,8 +276,8 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
         for line_num, line in enumerate(self.buf):
             if line_num == self.toc_marker and self.toc_marker > 0:
                 # Don't start ToC too close to the end of the page
-                if self.page_length + 10 >= max_page_length:
-                    remainder = max_page_length - self.page_length - 2
+                if self.pis['tocpagebreak'] == 'yes' or self.page_length + 10 >= max_page_length:
+                    remainder = max_page_length - self.page_length - self.page_end_blank_lines
                     self.emit([''] * remainder)
                     self.page_break()
 
@@ -264,7 +285,7 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
                 toc_prev_start = len(self.output)
                 preliminary_toc = self.write_toc(paging=True)
                 for l in preliminary_toc:
-                    if self.page_length + 2 >= max_page_length:
+                    if self.page_length + self.page_end_blank_lines >= max_page_length:
                         # Store a pair of TOC pointers
                         toc_pointers.append((toc_prev_start, len(self.output)))
                         # New page
@@ -278,7 +299,7 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
             if line_num == self.iref_marker and self.iref_marker > 0:
                 # Don't start Index too close to the end of the page
                 if self.page_length + 10 >= max_page_length:
-                    remainder = max_page_length - self.page_length - 2
+                    remainder = max_page_length - self.page_length - self.page_end_blank_lines
                     self.emit([''] * remainder)
                     self.page_break()
 
@@ -290,7 +311,7 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
                 iref_prev_start = len(self.output)
                 preliminary_iref_index = self.write_iref_index()
                 for l in preliminary_iref_index:
-                    if self.page_length + 2 >= max_page_length:
+                    if self.page_length + self.page_end_blank_lines >= max_page_length:
                         # Store a pair of pointers
                         iref_pointers.append((iref_prev_start, len(self.output)))
                         # New page
@@ -305,7 +326,7 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
                 # If this size hint exceeds the rest of the page, or is set
                 # to -1 (a forced break), insert a break.
 
-                available = max_page_length - (self.page_length + 2)
+                available = max_page_length - (self.page_length + self.page_end_blank_lines)
                 needed, text_type = self.break_hints[line_num]
 
                 blanks = 0
@@ -324,17 +345,17 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
                     needed -= blanks
 
                 if (text_type == "break"
-                    or (text_type == "raw" and needed > available and needed < max_page_length-2 )):
+                    or (text_type == "raw" and needed > available and needed < max_page_length-self.page_end_blank_lines )):
                     lines_until_break = 0
 
                 elif (self.pis['autobreaks'] == 'yes'
                       and needed > available
-                      and needed < max_page_length-2
-                      and (needed-available < 2 or available < 2) ):
+                      and needed < max_page_length-self.page_end_blank_lines
+                      and (needed-available < self.widow_limit or available < self.orphan_limit) ):
                     lines_until_break = available
                     if available == needed - 1:
                         lines_until_break -= 1
-                    if lines_until_break < 2:
+                    if lines_until_break < self.page_end_blank_lines:
                         lines_until_break = 0
                     if lines_until_break > 0 and blanks > 0:
                         # put back that blank line since we are going to emit it
@@ -342,12 +363,12 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
 
             if lines_until_break == 0:
                 # Insert break
-                remainder = max_page_length - self.page_length - 2
+                remainder = max_page_length - self.page_length - self.page_end_blank_lines
                 self.emit([''] * remainder)
             if not self.IsFormatting(line):
                 lines_until_break -= 1
 
-            if self.page_length + 2 >= max_page_length:
+            if self.page_length + self.page_end_blank_lines >= max_page_length:
                 # New page
                 self.page_break()
 
@@ -368,7 +389,7 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
 
         # Write final footer
         if self.page_length > 1:
-            remainder = max_page_length - self.page_length - 2
+            remainder = max_page_length - self.page_length - self.page_end_blank_lines
             self.emit([''] * remainder)
         self.page_break(final=True)
         
