@@ -4,9 +4,10 @@
 
 # Python libs
 import textwrap
-import lxml
 import datetime
+import lxml
 import re
+
 try:
     import debug
     assert debug
@@ -14,7 +15,7 @@ except ImportError:
     pass
 
 # Local lib
-from xml2rfc.writers.base import BaseRfcWriter
+from xml2rfc.writers.base import BaseRfcWriter, default_options
 import xml2rfc.utils
 
 
@@ -24,8 +25,10 @@ class RawTextRfcWriter(BaseRfcWriter):
         The page width is controlled by the *width* parameter.
     """
 
-    def __init__(self, xmlrfc, width=72, margin=3, quiet=False, verbose=False, date=datetime.date.today()):
-        BaseRfcWriter.__init__(self, xmlrfc, quiet=quiet, verbose=verbose, date=date)
+    def __init__(self, xmlrfc, width=72, margin=3, quiet=None, options=default_options, date=datetime.date.today()):
+        if not quiet is None:
+            options.quiet = quiet
+        BaseRfcWriter.__init__(self, xmlrfc, options=options, date=date)
         # Document processing data
         self.width = width      # Page width
         self.margin = margin    # Page margin
@@ -36,7 +39,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         self.list_counters = {} # Maintain counters for 'format' type lists
         self.edit_counter = 0   # Counter for edit marks
         # Set this to False to permit utf+8 output:
-        self.ascii = True       # Enable ascii flag
+        self.ascii = not options.utf8       # Enable ascii flag
         self.cref_counter = 0   # Counter for cref anchors
         self.cref_list = []
 
@@ -47,7 +50,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         self.inline_tags = ['xref', 'eref', 'iref', 'cref', 'spanx']
         
         # Custom textwrapper object
-        self.wrapper = xml2rfc.utils.MyTextWrapper(width=self.width,
+        self.wrapper = xml2rfc.utils.TextWrapper(width=self.width,
                                                    fix_sentence_endings=True)
 
     def _length(self, text):
@@ -95,7 +98,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         subsequent = ' ' * (indent + sub_indent)
         if bullet:
             initial = ' ' * indent
-            bullet_parts = self.wrapper.wrap(bullet, initial_indent=initial, subsequent_indent=initial,
+            bullet_parts = self.wrapper.wrap(bullet, initial=initial,
                                              fix_doublespace=False, drop_whitespace=False)
             if len(bullet_parts) > 1:
                 buf.extend(bullet_parts[:-1])
@@ -112,11 +115,11 @@ class RawTextRfcWriter(BaseRfcWriter):
                 # Strip initial whitespace
                 string = string.lstrip()
             if wrap_urls:
-                par = self.wrapper.wrap(string, initial_indent=initial, subsequent_indent=subsequent,
+                par = self.wrapper.wrap(string, initial=initial, subsequent_indent=subsequent,
                                         fix_sentence_endings=fix_sentence_endings,
                                         fix_doublespace=fix_doublespace)
             else:
-                par = self.wrapper.wrap(xml2rfc.utils.urlkeep(string), initial_indent=initial,
+                par = self.wrapper.wrap(xml2rfc.utils.urlkeep(string), initial=initial,
                                         subsequent_indent=subsequent,
                                         fix_sentence_endings=fix_sentence_endings,
                                         fix_doublespace=fix_doublespace)
@@ -163,7 +166,7 @@ class RawTextRfcWriter(BaseRfcWriter):
             # Check for optional hangIndent
             try:
                 hangIndent = list.attrib.get('hangIndent', None)
-                if hangIndent:
+                if hangIndent != None:
                     hangIndent = int(hangIndent)
             except (ValueError, TypeError):
                 xml2rfc.log.error("hangIndent value '%s' is not an integer" % hangIndent)
@@ -188,14 +191,16 @@ class RawTextRfcWriter(BaseRfcWriter):
         for element in list:
             # Check for PI
             if element.tag is lxml.etree.PI:
-                self.parse_pi(element)
+                pidict = self.parse_pi(element)
+                if pidict and "needLines" in pidict:
+                    self.needLines(pidict["needLines"])
             elif element.tag == 't':
                 # Disable linebreak if subcompact=yes AND not first list element
                 leading_blankline = True
-                if t_count > 0 and self.pis['subcompact'] == 'yes':
+                if t_count > 0 and element.pis['subcompact'] == 'yes':
                     leading_blankline = False
                 if style == 'symbols':
-                    bullet = self.pis['text-list-symbols'][level % len(self.pis['text-list-symbols'])]
+                    bullet = element.pis['text-list-symbols'][level % len(element.pis['text-list-symbols'])]
                     bullet += '  '
                 elif style == 'numbers':
                     bullet = self._format_counter("%d.", t_count+1, listlength)
@@ -204,7 +209,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                     bullet = self._format_counter(letter_style, t_count+1, listlength)
                 elif style == 'hanging':
                     bullet = element.attrib.get('hangText', '')
-                    if not hangIndent:
+                    if hangIndent is None:
                         hangIndent = 3
                     if len(bullet) < hangIndent:
                         # Insert whitespace up to hangIndent
@@ -214,7 +219,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                         bullet += '  '
                     # Add an extra space in front of colon if colonspace enabled
                     if bullet.endswith(':') and \
-                    self.pis['colonspace'] == 'yes':
+                    element.pis['colonspace'] == 'yes':
                         bullet+= ' '
                     if element.text and len(bullet) > self.width//2:
                         # extra check of remaining space if the bullet is
@@ -425,9 +430,6 @@ class RawTextRfcWriter(BaseRfcWriter):
         """
         line = ['']
         for i, element in enumerate(elements):
-            # Check for a PI first
-            if element.tag is lxml.etree.PI:
-                self.parse_pi(element)
             if element.tag not in self.inline_tags:
                 # Not an inline element, exit
                 return ''.join(line), elements[i:]
@@ -435,19 +437,18 @@ class RawTextRfcWriter(BaseRfcWriter):
             if element.tag == 'xref':
                 line.append(self._expand_xref(element))
             elif element.tag == 'eref':
-                if element.text:
-                    line.append(element.text + ' ')
+                if element.text and len(element.text)>0:
+                    line.append(element.text.strip())
                     self.eref_count += 1
-                    if element.text != element.attrib['target']:
-                        line.append('[' + str(self.eref_count) + ']')
-                        if self.indexmode:
-                            self.eref_list.append([self.eref_count, element])
+                    line.append(' [' + str(self.eref_count) + ']')
+                    if self.indexmode:
+                        self.eref_list.append([self.eref_count, element])
                 else:
-                    line.append('<' + element.attrib['target'] + '>')
+                    line.append('<' + element.attrib['target'].strip() + '>')
             elif element.tag == 'iref':
                 self._add_iref_to_index(element)
             elif element.tag == 'cref' and \
-                self.pis['comments'] == 'yes':                
+                element.pis['comments'] == 'yes':                
                 # Render if processing instruction is enabled
                 self.cref_counter += 1
                 anchor = element.attrib.get('anchor', None)
@@ -455,7 +456,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                     anchor = 'CREF' + str(self.cref_counter)
                     element.attrib['anchor'] = anchor
                 self._indexCref(self.cref_counter, anchor)
-                if self.pis['inline'] == 'yes':
+                if element.pis['inline'] == 'yes':
                     if anchor:
                         anchor = anchor + ': '
                     source = element.attrib.get('source', "")
@@ -468,7 +469,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                     self.cref_list.append(element)
             elif element.tag == 'spanx':
                 style = element.attrib.get('style', 'emph')
-                edgechar = '?'
+                edgechar = '_'  # default to emph because the spanx element exists
                 if style == 'emph':
                     edgechar = '_'
                 elif style == 'strong':
@@ -576,7 +577,7 @@ class RawTextRfcWriter(BaseRfcWriter):
     def write_title(self, title, docName=None, source_line=None):
         """ Write the document title and (optional) name """
         self.write_text(title, leading_blankline=True, align='center', source_line=source_line)
-        if docName:
+        if docName and not self.rfcnumber:
             self.write_text(docName, align='center')
 
     def write_heading(self, text, bullet='', autoAnchor=None, anchor=None, \
@@ -733,6 +734,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         refkeys = []
         refsource = {}
         # [surname, initial.,] "title", (STD), (BCP), (RFC), (Month) Year.
+        i = 0
         for i, ref in enumerate(list.findall('reference')):
             refstring = []
             authors = ref.findall('front/author')
@@ -853,7 +855,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         for i, cell in enumerate(table.findall('c')):
             if i % num_columns == 0:
                 # Insert blank row if PI 'compact' is 'no'
-                if self.pis["compact"] == "no":
+                if table.pis["compact"] == "no":
                     if (style == 'none' and row == 0) or (style in ['headers', 'full'] and row != 0):
                         row += 1
                         matrix.append(['']*num_columns)
@@ -890,9 +892,10 @@ class RawTextRfcWriter(BaseRfcWriter):
                     if self._length(row[col]) > longest_lines[col]:
                         longest_lines[col] = self._length(row[col])
                     # Longest word
-                    word = max(xml2rfc.utils.ascii_split(row[col]), key=self._length)
-                    if self._length(word) > longest_words[col]:
-                        longest_words[col] = self._length(word)
+                    if row[col].strip():
+                        word = max(xml2rfc.utils.ascii_split(row[col]), key=self._length)
+                        if self._length(word) > longest_words[col]:
+                            longest_words[col] = self._length(word)
         
         min_width = sum(longest_words)
         
@@ -1421,6 +1424,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 # line = line.replace(u"\u8209", '-')
                 line = line.replace(u'\u00A0', u' ')
                 line = line.replace(u'\u2011', u'-')
+                line = line.replace(u'\u200B', u'')
             output.append(line);
         return output
 
